/*
 *  Copyright (C) 2003 Marco Pesenti Gritti
 *  Copyright (C) 2003 Christian Persch
 *  Copyright (C) 2004 Crispin Flowerday
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 */

#include "mozilla-helpers.h"

#include <nsCOMPtr.h>
#include <nsIWebBrowser.h>
#include <nsIDocShell.h>
#include <nsIWebPageDescriptor.h>
#include <nsIInterfaceRequestorUtils.h>
#include <nsIWebNavigation.h>

#include <gtkmozembed.h>
#include <gtkmozembed_internal.h>


static nsresult
GetPageDescriptor (EphyEmbed *aEmbed, nsISupports ** aPageDescriptor)
{
	NS_ENSURE_TRUE (EPHY_IS_EMBED (aEmbed), NS_ERROR_FAILURE);
	
	nsCOMPtr<nsIWebBrowser> browser;
        gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED (aEmbed),
                                         getter_AddRefs (browser));

	nsCOMPtr<nsIDocShell> ds = do_GetInterface (browser);
	NS_ENSURE_TRUE (ds, NS_ERROR_FAILURE);

	nsCOMPtr<nsIWebPageDescriptor> wpd = do_QueryInterface (ds);
	NS_ENSURE_TRUE (wpd, NS_ERROR_FAILURE);

	return wpd->GetCurrentDescriptor(aPageDescriptor);
}


static nsresult
ViewPageSource (EphyEmbed *aEmbed, nsISupports * aPageDescriptor)
{
	NS_ENSURE_TRUE (EPHY_IS_EMBED (aEmbed), NS_ERROR_FAILURE);
	NS_ENSURE_TRUE (aPageDescriptor, NS_ERROR_FAILURE);

	nsCOMPtr<nsIWebBrowser> browser;
        gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED (aEmbed),
                                         getter_AddRefs (browser));
	
	nsCOMPtr<nsIDocShell> ds = do_GetInterface (browser);
	NS_ENSURE_TRUE (ds, NS_ERROR_FAILURE);

	nsCOMPtr<nsIWebPageDescriptor> wpd = do_QueryInterface (ds);
	NS_ENSURE_TRUE (wpd, NS_ERROR_FAILURE);

	return wpd->LoadPage(aPageDescriptor, nsIWebPageDescriptor::DISPLAY_AS_SOURCE);
}


extern "C" void
mozilla_view_source (EphyEmbed *aOldEmbed, EphyEmbed *aNewEmbed)
{
	g_return_if_fail (EPHY_IS_EMBED (aOldEmbed));
	g_return_if_fail (EPHY_IS_EMBED (aNewEmbed));

        nsCOMPtr<nsISupports> pageDescriptor;
	GetPageDescriptor (aOldEmbed, getter_AddRefs (pageDescriptor));

	ViewPageSource (aNewEmbed, pageDescriptor);
}


extern "C" void
mozilla_reload_source (EphyEmbed *aEmbed)
{
	g_return_if_fail (EPHY_IS_EMBED (aEmbed));

	nsCOMPtr<nsIWebBrowser> browser;
        gtk_moz_embed_get_nsIWebBrowser (GTK_MOZ_EMBED (aEmbed),
                                         getter_AddRefs (browser));

	/* Don't use gtkmozembed's reload routine at the moment,
	 * it has bugs all of my own making:
	 *
	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
	 */
	nsCOMPtr<nsIWebNavigation> wn = do_QueryInterface (browser);
	if (wn)
	{
		wn->Reload (nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
			    nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY);
	}
}
